import xlrd
import xlwt
from xlwt import Pattern, Style, XFStyle


class OperationExcel:
    def __init__(self, file_name, sheet_name):
        """

        :param file_name: 路径+文件名
        :param sheet_name: 工作表名
        """
        self.workbook = xlrd.open_workbook(file_name)
        self.sheet = self.workbook.sheet_by_name(sheet_name)

    def get_value(self, row, col):
        sheet_value = self.sheet.cell_value(row-1, col-1)
        return sheet_value

    def get_values(self, start_row, end_row, start_col, end_col):
        results = []
        if start_row < end_row and start_col < end_col:
            for item in range(start_row-1, end_row):
                results.append(self.sheet.row_values(item, start_col-1, end_col))
        elif start_row < end_row and start_col == end_col:
            results.append(self.sheet.col_values(start_col-1, start_row-1, end_row))
        elif start_row == end_row and start_col < end_col:
            results.append(self.sheet.row_values(start_row-1, start_col-1, end_col))
        elif start_row == end_row and start_col == end_col:
            results = self.get_value(start_row, start_col)
        else:
            raise ValueError(f'start_row must ≤ end_row and start_col must  ≤ end_col ')
        return results


class TestExcel:
    def __init__(self, file_name, sheet_name):
        """

        :param file_name: 文件地址+文件名
        :param sheet_name: 工作表名
        """
        self.file_name = file_name
        self.book = xlwt.Workbook()  # 新建工作簿
        self.sheet = self.book.add_sheet(sheet_name)  # 新建sheet

    @staticmethod
    def set_cell_style(color, font_name='宋体', font_bold=False, font_italic=False, font_underline=False):
        # 颜色处理
        new_pattern = Pattern()
        new_pattern.pattern = Pattern.SOLID_PATTERN
        new_pattern.pattern_fore_colour = Style.colour_map[color]
        new_style = XFStyle()
        new_style.pattern = new_pattern
        # 字体处理
        font = xlwt.Font()
        font.name = font_name  # 字体名称
        font.bold = font_bold  # 加粗
        font.italic = font_italic  # 斜体
        font.underline = font_underline  # 下划线
        new_style.font = font
        return new_style

    def set_value(self, row, col, body, color='white', font_name='宋体', font_bold=False, font_italic=False,
                  font_underline=False):
        style = self.set_cell_style(color, font_name, font_bold, font_italic, font_underline)
        self.sheet.write(row - 1, col - 1, body, style)
        self.book.save(self.file_name)

    def set_values(self, row, col, values, headers='', color='red',
                   font_name='宋体', font_bold=False, font_italic=False, font_underline=False):
        """

        :param row:
        :param col:
        :param values: [(),()...]
        :param headers: {}
        :param color:
        :param font_name: 字体名称
        :param font_bold: 加粗
        :param font_italic: 斜体
        :param font_underline: 下划线
        :return:
        """
        row, col = row - 1, col - 1
        style = self.set_cell_style(color, font_name, font_bold, font_italic, font_underline)
        if headers == '':
            for index, item in enumerate(values):
                for index_col, item_col in enumerate(item):
                    self.sheet.write(row + index, col + index_col, item_col, style)
                    self.book.save(self.file_name)
        else:
            for index_h, item_h in enumerate(headers):
                self.sheet.write(0, index_h, item_h)
            for index_v, item_v in enumerate(values):
                for index_col, item_col in enumerate(item_v):
                    self.sheet.write(row + index_v, col + index_col, item_col, style)
                    self.book.save(self.file_name)



import base64
import hashlib
import hmac

"""
@description: 加密方法
"""


class Encryption:
    def __init__(self, not_encrypted_str, mode, key=''):
        """

        :param not_encrypted_str: 未加密字符串
        :param mode: 加密方式 md5、sha256...
        :param key: 盐 可选
        """
        self.mode = mode  # Es = Encrypted string
        self.key = key
        self.NES = not_encrypted_str

    def encryption(self):
        if self.key == '':
            encrypted_string = hashlib.new(self.mode, bytes(self.NES, encoding='utf-8')).hexdigest()
            return encrypted_string

        else:
            encrypted_string = hmac.new(
                bytes(self.key, encoding='utf-8'), bytes(self.NES, encoding='utf-8'), self.mode
            )
            return encrypted_string.hexdigest()

        """
        Example 1

        esd = Encryption('123456', 'md5', 'key').encryption()
        print(esd)

        Example 2
        esd = Encryption('123456', 'md5').encryption()
        """


def to_base64(item):
    """

    :param item: str
    :return: str.base64
    """
    return str(base64.b64encode(item.encode('utf-8')), "utf-8")



import sys
import os


class PictureSynthesis:
    """
        图片合成  图片a=100k file(b)=10M  new_file>10M
    """

    def __init__(self, img_file, other_file, new_file):
        self.old_img = img_file
        self.big_file = other_file
        self.new_img = new_file

    def picture_synthesis(self):
        os_name = sys.platform
        if os_name == 'linux' or os_name == 'darwin':
            os.system(f'cat {self.old_img} {self.big_file} >{self.new_img}')
        elif os_name == 'win32':
            os.system(f'copy /b {self.old_img}+{self.big_file} {self.new_img} ')
        else:
            print('此系统暂不支持')



import csv
import logging

logging.basicConfig(level=logging.ERROR)


class OperationCSV:
    def __init__(self, path):
        self.path = path

    def get_value(self, row_num, col_num):
        """

        :param row_num:
        :param col_num:
        :return: csv_value
        """
        with open(self.path, 'r') as file:
            reader = csv.reader(file)
            rows = [row for row in reader]
            return rows[row_num][col_num]

    def get_values(self, row_start, row_end, col_start, cor_end):
        """

        :param row_start: 开始行
        :param row_end: 结束行
        :param col_start: 开始列
        :param cor_end: 结束列
        :return: 查询结果
        """
        # 物理数改为索引
        row_start = row_start - 1
        row_end = row_end - 1
        col_start = col_start - 1
        cor_end = cor_end - 1
        with open(self.path, 'r') as file:
            reader = csv.reader(file)
            results = []
            for index, info in enumerate(reader):   # 获取索引和对应数据
                if row_start < row_end and col_start < cor_end:
                    if row_start <= index <= row_end:
                        results.append(info[col_start:cor_end + 1])
                elif row_start < row_end and col_start == cor_end:
                    if row_start <= index <= row_end:
                        results.append(info[col_start:col_start + 1])
                elif row_start == row_end and col_start < cor_end:
                    if row_start == index:
                        results.append(info[col_start: cor_end + 1])
                elif row_start == row_end and col_start == cor_end:
                    if row_start == index:
                        results.append(self.get_value(row_start, col_start))
                else:
                    if row_start > row_end and col_start > cor_end:
                        return logging.error('row_start>row_end and col_start > cor_end')
                    elif col_start > cor_end:
                        return logging.error('col_start>cor_end')
                    else:
                        return logging.error('row_start > row_end')
            return results

    def write_value(self, values, headers=None):
        """

        :param values: [(),()...]
        :param headers: dict
        :return:
        """
        if headers is None:
            with open(self.path, 'w', newline='') as file:
                f_write = csv.writer(file)
                f_write.writerows(values)
        elif headers is not None:
            with open(self.path, 'w', newline='') as file:
                f_write = csv.writer(file)
                f_write.writerow(headers)
                f_write.writerows(values)



import pymysql


class Mysql:
    def __init__(self, host, user, password, database, charset="utf8"):
        self.conn = pymysql.connect(
            host=host,
            user=user,
            password=password,
            database=database,
            charset=charset,
        )
        self.cursor = self.conn.cursor(cursor=pymysql.cursors.DictCursor)  # 得到一个可以执行SQL语句的光标对象 结果返回字典 默认元祖

    def close_db(self):
        self.cursor.close()

    def data_operation(self, sql, data='', operation='select'):
        """

        :param sql:
        :param data: Usually used when inserting data; [(),(),...]
                    when there is no data，（sql, '', operation）
        :param operation: default is select
        :return:
        """
        if data == '':
            if operation != 'select':
                self.cursor.execute(sql)
                self.conn.commit()
                self.cursor.close()
            else:
                self.cursor.execute(sql)
                results = self.cursor.fetchall()
                self.cursor.close()
                return results
        elif data != '':
            self.cursor.executemany(sql, data)
            self.conn.commit()
            self.cursor.close()


'''
    Example 1
    s = Mysql('host', 'user', 'pwd', 'database')
    s1 = s.data_operation('select * from user', '', 'select')
    print(s1)

    Example 2
    s = Mysql('host', 'user', 'pwd', 'database')
    data = [
        ('july', '147'),
        ('june', '258'),
        ('marin', '369')
    ]
    s1 = s.data_operation('insert into userinfo(user,pwd) values(%s,%s)', data)

    '''


import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText


class SendMail:
    def __init__(self, server, port, email, password):
        """

        :param server: 服务器主机
        :param port:    端口
        :param email:
        :param password:
        """
        self.smtp = smtplib.SMTP()
        self.smtp.connect(server, port)
        self.smtp.login(email, password)

    def send_mail(self, file, form_mail, to_mail, filename, subject, body):  # filename 不要用中文
        """
        :param file: 文件
        :param form_mail: The address sending this mail.
        :param to_mail: A list of addresses to send this mail to.A bare string will be treated as a list with 1 address.
        :param filename: Attachment name
        :param subject:
        :param body:
        :return:
        """

        sendfile = open(file, 'rb').read()
        att = MIMEText(sendfile, 'base64', 'utf-8')
        att['Content-Type'] = 'application/octet-stream'
        att['Content-Disposition'] = 'attachment; filename= ' + filename  # 附件名称
        msg = MIMEMultipart()
        msg['Subject'] = subject  # '主题'
        msg.attach(MIMEText(body))  # 正文
        msg.attach(att)  # 附件
        try:
            self.smtp.sendmail(form_mail, to_mail, msg.as_string())  # 参数 1: 发送的邮箱， 2 接收的邮箱
            self.smtp.quit()
            return '邮件发送成功'
        except smtplib.SMTPException:
            self.smtp.quit()
            return '邮件发送失败'


"""
    mail = SendMail('smtp.163.com', 25, 'mail', 'password')
    file1 = os.path.join('/a/b/c', 'd.xls')
    mail.send_mail(file1, 'from_mail', 'to_mail', 'd.xls', '主题', '正文')
"""



import random
import datetime
import time


def phone_num(them_roughly: str = '000'):
    if them_roughly == '000':
        them_roughly_list = ["130", "131", "132", "133", "134", "135", "136", "137", "138", "139", "147", "150", "151",
                             "152", "153", "155", "156", "157", "158", "159", "186", "187", "188", "189", "172", "176",
                             "185"]
        phone = random.choice(them_roughly_list) + "".join(random.choice("0123456789") for completion in range(8))

    else:
        phone = str(them_roughly) + "".join(random.choice("0123456789") for completion in range(8))
    return phone


def id_card_num(address: str = '000000', birthday: str = '00000000'):
    end_num = "".join(random.choice("0123456789") for completion in range(4))
    start_time = datetime.datetime(1900, 1, 1).timestamp()
    end_time = datetime.datetime(datetime.datetime.now().year, datetime.datetime.now().month,
                                 datetime.datetime.now().day).timestamp()
    if address == '000000':
        address = str(random.randint(110000, 900000))
        if birthday == '00000000':
            birthday = random.randint(start_time, end_time)
            birthday = time.localtime(birthday)
            birthday = str(time.strftime("%Y%m%d", birthday))
            result = address + birthday + str(end_num)
        else:
            result = address + birthday + end_num
    else:
        if birthday == '000000':
            birthday = random.randint(start_time, end_time)
            birthday = time.localtime(birthday)
            birthday = str(time.strftime("%Y%m%d", birthday))
            result = address + birthday + end_num
        else:
            result = address + birthday + end_num

    return result


def name_chinese(length: int = 2):
    result = ''
    for times in range(length):
        result = result + chr(random.randint(0x4e00, 0x9fbf))
    return result


__all__ = [
    'phone_num',
    'id_card_num',
    'name_chinese'
]



import os


def timing_task_add(task_time, password):
    """ task_time: Cron expression,
        password: linux root_password
    """
    os.system(f'echo "{password}" | sudo -S sed -i "\$a{task_time}" /etc/crontab')
    os.system(f'server cron reload')
    os.system('cat /etc/crontab')


__all__ = [
    "timing_task_add"
]



import base64
import os


def img_to_base64(image_bytes):
    """

    :param image_bytes: 文件路径
    :return: base64字符串
    """
    with open(image_bytes, "rb") as fb:
        img = fb.read()
        base64_bytes = base64.b64encode(img)
        img_suffix = os.path.splitext(image_bytes)[1].replace('.', '')
        if img_suffix == 'jpg' or img_suffix == 'JPG' or img_suffix == 'jpeg' or img_suffix == 'JPEG':
            base64_str = 'data:image/jpeg;base64,' + base64_bytes.decode("utf-8")
        elif img_suffix == 'png' or img_suffix == 'PNG':
            base64_str = 'data:image/png;base64,' + base64_bytes.decode("utf-8")
        elif img_suffix == 'gif' or img_suffix == 'GIF':
            base64_str = 'data:image/gif;base64,' + base64_bytes.decode("utf-8")
        elif img_suffix == 'icon' or img_suffix == 'ICON':
            base64_str = 'data:image/x-icon;base64,' + base64_bytes.decode("utf-8")
        else:
            return '不支持此格式文件'
    return base64_str



from distutils.core import setup
from setuptools import find_packages

with open("README.md", 'rb') as f:
    long_description = f.read()

setup(name='ruffianTest',  # 你包的名称
      version='0.0.2',  # 版本号
      description='test',  # 描述
      long_description=long_description,  # 长描述
      long_description_content_type='text/markdown',
      author='The-Ruffian',
      author_email='',
      url='https://github.com/the-ruffian/ruffian_test',
      download_url='https://github.com/the-ruffian/ruffian_test',
      install_requires=['PyMySQL', 'xlrd', 'xlwt'],  # 依赖第三方库
      license='MIT License',
      keywords=['bug_pz', 'the-ruffian', 'ruffianTest', 'ruffian', 'test'],
      packages=find_packages(),
      platforms=['all'],  # 平台
      classifiers=[
          'Intended Audience :: Developers',
          'Operating System :: OS Independent',
          'Natural Language :: Chinese (Simplified)',
          'Programming Language :: Python :: 3',
          'Topic :: Software Development :: Testing',
          'Development Status :: 5 - Production/Stable'
      ],
      )
